@@ -336,11 +336,11 @@ makeClusterPSOCK <- function(workers, makeNode = makeNodePSOCK, port = c("auto",
 #' system executable for this is given by argument `rshcmd`.  The default
 #' is given by option \option{parallelly.makeNodePSOCK.rshcmd}.  If that is not
 #' set, then the default is to use \command{ssh} on Unix-like systems,
-#' including macOS.  On MS Windows systems, including Windows 10, the
-#' default is to use (i) \command{plink} from the
-#' \href{https://www.putty.org/}{\command{PuTTY}} project,
-#' (ii) the \command{ssh} client that is distributed with RStudio, and lastly
-#' (iii) the \command{ssh} client that comes with Windows 10.
+#' including macOS as well as Windows 10.  On older MS Windows versions, which
+#' does not have a built-in \command{ssh} client, the default is to use
+#' (i) \command{plink} from the \href{https://www.putty.org/}{\command{PuTTY}}
+#' project, and then (ii) the \command{ssh} client that is distributed with
+#' RStudio.
 #'
 #' PuTTY puts itself on Windows' system \env{PATH} when installed, meaning this
 #' function will find PuTTY automatically if installed.  If not, to manually
@@ -366,16 +366,6 @@ makeClusterPSOCK <- function(workers, makeNode = makeNodePSOCK, port = c("auto",
 #'
 #' If no SSH-client is found, an informative error message is produced.
 #'
-#' (*) _Windows 10 has a \command{ssh} built-in since version 1803 (April 2018).
-#' However, there is a bug in that SSH client that prevents it to work with
-#' reverse SSH tunneling
-#' (\url{https://github.com/PowerShell/Win32-OpenSSH/issues/1265}; Oct 2018).
-#' The most recent version that we tested and that did _not_ work was
-#' OpenSSH_for_Windows_7.7p1, LibreSSL 2.6.5 (`ssh -V`) on
-#' Windows 10 (version 1909, OS build 18363.720) (`ver`).
-#' Because of this, it is recommended to use the PuTTY SSH client or the
-#' RStudio SSH client until this bug has been resolved in Windows 10._
-#' 
 #' Additional SSH options may be specified via argument `rshopts`, which
 #' defaults to option \option{parallelly.makeNodePSOCK.rshopts}. For instance, a
 #' private SSH key can be provided as
@@ -580,7 +570,7 @@ makeNodePSOCK <- function(worker = "localhost", master = NULL, port, connectTime
 
   if (is.null(master)) {
     if (localMachine || revtunnel) {
-      master <- "localhost"
+      master <- getOptionOrEnvVar("parallelly.localhost.hostname", "localhost")
     } else {
       master <- Sys.info()[["nodename"]]
     }
@@ -737,7 +727,7 @@ makeNodePSOCK <- function(worker = "localhost", master = NULL, port, connectTime
     }
     ## Any environment variables to set?
     if (length(names) > 0L) {
-      code <- sprintf('%s="%s"', names, rscript_envs)
+      code <- sprintf('"%s"="%s"', names, rscript_envs)
       code <- paste(code, collapse = ", ")
       code <- paste0("Sys.setenv(", code, ")")
       tryCatch({
@@ -813,6 +803,10 @@ makeNodePSOCK <- function(worker = "localhost", master = NULL, port, connectTime
       if (is.null(attr(rshcmd, "type"))) attr(rshcmd, "type") <- "<unknown>"
       if (is.null(attr(rshcmd, "version"))) attr(rshcmd, "version") <- "<unknown>"
     }
+    
+    ## Holds a pathname with an optional set of command-line options
+    stop_if_not(is.character(rshcmd), length(rshcmd) >= 1L)
+    
     s <- sprintf("type=%s, version=%s", sQuote(attr(rshcmd, "type")), sQuote(attr(rshcmd, "version")))
     rshcmd_label <- sprintf("%s [%s]", paste(sQuote(rshcmd), collapse = ", "), s)
 
@@ -823,15 +817,15 @@ makeNodePSOCK <- function(worker = "localhost", master = NULL, port, connectTime
 
     ## Reverse tunneling?
     if (revtunnel) {
-      rshopts <- c(sprintf("-R %d:%s:%d", rscript_port, master, port), rshopts)
-      ## AD HOC: Warn about Windows 10 SSH bug with rev tunneling
-      if (isTRUE(attr(rshcmd, "OpenSSH_for_Windows"))) {
-         ver <- windows_build_version()
-         if (!is.null(ver) && ver <= "10.0.17763.253") {
-           msg <- sprintf("WARNING: You're running Windows 10 (build %s) where this 'rshcmd' (%s) may not support reverse tunneling (revtunnel = TRUE) resulting in worker failing to launch", ver, paste(sQuote(rshcmd), collapse = ", "), rshcmd_label)
-           if (verbose) message(c(verbose_prefix, msg))
-         }
+      ## WORKAROUND: Avoid revtunnel bug in Windows 10's ssh client:
+      ## https://github.com/PowerShell/Win32-OpenSSH/issues/1265#issuecomment-637085238
+      if (master == "localhost" && .Platform$OS.type == "windows" && (
+           isTRUE(attr(rshcmd, "OpenSSH_for_Windows")) ||
+           basename(rshcmd[1]) == "ssh"
+         )) {
+        master <- "127.0.0.1"
       }
+      rshopts <- c(sprintf("-R %d:%s:%d", rscript_port, master, port), rshopts)
     }
     
     ## SSH log file?
@@ -990,7 +984,7 @@ makeNodePSOCK <- function(worker = "localhost", master = NULL, port, connectTime
 
        ## Log file?
        if (is.character(rshlogfile)) {
-         smsg <- sprintf("Inspect the content of log file %s for %s.", sQuote(rshlogfile), sQuote(rshcmd))
+         smsg <- sprintf("Inspect the content of log file %s for %s.", sQuote(rshlogfile), paste(sQuote(rshcmd), collapse = " "))
          lmsg <- tryCatch(readLines(rshlogfile, n = 15L, warn = FALSE), error = function(ex) NULL)
          if (length(lmsg) > 0) {
            lmsg <- sprintf("     %2d: %s", seq_along(lmsg), lmsg)
@@ -998,7 +992,7 @@ makeNodePSOCK <- function(worker = "localhost", master = NULL, port, connectTime
          }
          suggestions <- c(suggestions, smsg)
        } else {
-         suggestions <- c(suggestions, sprintf("Set 'rshlogfile=TRUE' to enable logging for %s.", sQuote(rshcmd)))
+         suggestions <- c(suggestions, sprintf("Set 'rshlogfile=TRUE' to enable logging for %s.", paste(sQuote(rshcmd), collapse = " ")))
        }
        
        ## Special: Windows 10 ssh client may not support reverse tunneling. /2018-11-10
@@ -1130,6 +1124,9 @@ is_fqdn <- function(worker) {
 #' is produced, otherwise only a warning.
 #'
 #' @return A named list of pathnames to all located SSH clients.
+#' The pathnames may be followed by zero or more command-line options,
+#' i.e. the elements of the returned list are character vectors of length
+#' one or more.
 #' If `first = TRUE`, only the first one is returned.
 #' Attribute `version` contains the output from querying the
 #' executable for its version (via command-line option `-V`).
@@ -1138,7 +1135,9 @@ is_fqdn <- function(worker) {
 find_rshcmd <- function(which = NULL, first = FALSE, must_work = TRUE) {
   query_version <- function(bin, args = "-V") {
     v <- suppressWarnings(system2(bin, args = args, stdout = TRUE, stderr = TRUE))
-    paste(v, collapse = "; ")
+    v <- paste(v, collapse = "; ")
+    stop_if_not(length(v) == 1L)
+    v
   }
   
   find_rstudio_ssh <- function() {
@@ -1186,16 +1185,7 @@ find_rshcmd <- function(which = NULL, first = FALSE, must_work = TRUE) {
 
   if (is.null(which)) {
     if (.Platform$OS.type == "windows") {
-      which <- c("putty-plink", "rstudio-ssh")
-      ## Reverse tunnelling on SSH is not supported on Windows 10 with:
-      ## * OpenSSH_for_Windows_???, LibreSSL ???
-      ##   - Windows 10 version 1803 build 17134.523
-      ## * OpenSSH_for_Windows_7.7p1, LibreSSL 2.6.5
-      ##   - Windows 10 version 1809 build 17763.253
-      ##   - Windows 10 version 1903 build 18362.720
-      ##   - Windows 10 version 1909 build 18363.720
-      ## So it's unlikely that this will work out of the box.
-      which <- c(which, "ssh")
+      which <- c("ssh", "putty-plink", "rstudio-ssh")
     } else {
       which <- c("ssh")
     }
