@@ -37,6 +37,8 @@
 #' 
 #'  \item{\option{parallelly.availableCores.system} / \option{future.availableCores.system}:}{(integer) Number of "system" cores used instead of what is reported by \code{\link{availableCores}(which = "system")}. If not specified, this option is set according to system environment variable \env{R_PARALLELLY_AVAILABLECORES_SYSTEM} when the \pkg{parallelly} package is _loaded_. This option allows you to effectively override what `parallel::detectCores()` reports the system has.}
 #'
+#'  \item{\option{parallelly.availableCores.omit} / \option{future.availableCores.omit}:}{(integer) Number of cores to set aside, i.e. not to include.  If not specified, this option is set according to system environment variable \env{R_PARALLELLY_AVAILABLECORES_OMIT} when the \pkg{parallelly} package is _loaded_.}
+#'
 #'  \item{\option{parallelly.availableWorkers.methods} / \option{future.availableWorkers.methods}:}{(character vector) Default lookup methods for [availableWorkers()]. (Default: `c("mc.cores", "_R_CHECK_LIMIT_CORES_", "PBS", "SGE", "Slurm", "LSF", "custom", "system", "fallback")`)}
 #'
 #'  \item{\option{parallelly.availableWorkers.custom} / \option{future.availableWorkers.custom}:}{(function) If set and a function, then this function will be called (without arguments) by [availableWorkers()] where its value, coerced to a character vector, is interpreted as hostnames of available workers.}
@@ -62,6 +64,7 @@
 #' parallelly.availableCores.custom
 #' parallelly.availableCores.methods
 #' parallelly.availableCores.fallback R_PARALLELLY_AVAILABLECORES_FALLBACK
+#' parallelly.availableCores.omit R_PARALLELLY_AVAILABLECORES_OMIT
 #' parallelly.availableCores.system R_PARALLELLY_AVAILABLECORES_SYSTEM
 #' parallelly.availableWorkers.methods
 #' parallelly.fork.enable R_PARALLELLY_FORK_ENABLE
@@ -78,3 +81,107 @@
 #' @keywords internal
 #' @name parallelly.options
 NULL
+
+
+get_package_option <- function(name, default = NULL, package = .packageName) {
+  if (!is.null(package)) {
+    name <- paste(package, name, sep = ".")
+  }
+  getOption(name, default = default)
+}
+
+# Set an R option from an environment variable
+update_package_option <- function(name, mode = "character", default = NULL, package = .packageName, split = NULL, trim = TRUE, disallow = c("NA"), force = FALSE, debug = FALSE) {
+  if (!is.null(package)) {
+    name <- paste(package, name, sep = ".")
+  }
+
+  mdebugf("Set package option %s", sQuote(name))
+
+  ## Already set? Nothing to do?
+  value <- getOption(name, NULL)
+  if (!force && !is.null(value)) {
+    mdebugf("Already set: %s", sQuote(value))
+    return(getOption(name))
+  }
+
+  ## name="Pkg.foo.Bar" => env="R_PKG_FOO_BAR"
+  env <- gsub(".", "_", toupper(name), fixed = TRUE)
+  env <- paste("R_", env, sep = "")
+
+  env_value <- value <- Sys.getenv(env, unset = NA_character_)
+  if (is.na(value)) {  
+    if (debug) mdebugf("Environment variable %s not set", sQuote(env))
+    
+    ## Nothing more to do?
+    if (is.null(default)) return(getOption(name))
+
+    if (debug) mdebugf("Use argument 'default': ", sQuote(default))
+    value <- default
+  }
+
+  if (debug) mdebugf("%s=%s", env, sQuote(value))
+
+  ## Trim?
+  if (trim) value <- trim(value)
+
+  ## Nothing to do?
+  if (!nzchar(value)) return(getOption(name, default = default))
+
+  ## Split?
+  if (!is.null(split)) {
+    value <- strsplit(value, split = split, fixed = TRUE)
+    value <- unlist(value, use.names = FALSE)
+    if (trim) value <- trim(value)
+  }
+
+  ## Coerce?
+  mode0 <- storage.mode(value)
+  if (mode0 != mode) {
+    suppressWarnings({
+      storage.mode(value) <- mode
+    })
+    if (debug) {
+      mdebugf("Coercing from %s to %s: %s", mode0, mode, commaq(value))
+    }
+  }
+
+  if (length(disallow) > 0) {
+    if ("NA" %in% disallow) {
+      if (any(is.na(value))) {
+        stop(sprintf("Coercing environment variable %s=%s to %s would result in missing values for option %s: %s", sQuote(env), sQuote(env_value), sQuote(mode), sQuote(name), commaq(value)))
+      }
+    }
+    if (is.numeric(value)) {
+      if ("non-positive" %in% disallow) {
+        if (any(value <= 0, na.rm = TRUE)) {
+          stop(sprintf("Environment variable %s=%s specifies a non-positive value for option %s: %s", sQuote(env), sQuote(env_value), sQuote(name), commaq(value)))
+        }
+      }
+      if ("negative" %in% disallow) {
+        if (any(value < 0, na.rm = TRUE)) {
+          stop(sprintf("Environment variable %s=%s specifies a negative value for option %s: %s", sQuote(env), sQuote(env_value), sQuote(name), commaq(value)))
+        }
+      }
+    }
+  }
+  
+  if (debug) {
+    mdebugf("=> options(%s = %s) [n=%d, mode=%s]",
+            dQuote(name), commaq(value),
+            length(value), storage.mode(value))
+  }
+
+  do.call(options, args = structure(list(value), names = name))
+  
+  getOption(name, default = default)
+}
+
+
+## Set package options based on environment variables
+update_package_options <- function(debug = FALSE) {
+  update_package_option("availableCores.fallback", mode = "integer", disallow = NULL, debug = debug)
+  update_package_option("availableCores.system", mode = "integer", disallow = NULL, debug = debug)
+  update_package_option("availableCores.logical", mode = "logical", debug = debug)
+  update_package_option("availableCores.omit", mode = "integer", debug = debug)
+}
